package de.unisb.prog.typestate;

/**
 * Default and simple implementation of {@link IAnalysisConfiguration}.
 * 
 * @author mikerofone
 * 
 */
public class AnalysisConfiguration implements IAnalysisConfiguration {

	private TopStateMode			topStateMode;
	private UnknownTransitionMode	unknownTransMode;
	private int						maxDepth;
	private boolean					logNoSpec;
	private boolean	walkerUseOnlyOneWorklist;

	/**
	 * Default constructor creates a most conservative configuration.
	 */
	public AnalysisConfiguration() {
		topStateMode = TopStateMode.ASSUME_EVERY_STATE;
		unknownTransMode = UnknownTransitionMode.ERROR_TRANSITION;
		maxDepth = 1;
		logNoSpec = false;
		walkerUseOnlyOneWorklist = false;
	}

	public AnalysisConfiguration(TopStateMode topStateMode, UnknownTransitionMode unknownTransMode,
			int maxDepth, boolean logNoSpec, boolean useOnlyOneWorklist) {
		this.topStateMode = topStateMode;
		this.unknownTransMode = unknownTransMode;
		this.maxDepth = maxDepth;
		this.logNoSpec = logNoSpec;
		this.walkerUseOnlyOneWorklist = useOnlyOneWorklist;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IAnalysisConfiguration#getTopStateMode()
	 */
	public TopStateMode getTopStateMode() {
		return topStateMode;
	}

	/**
	 * @param topStateMode
	 *            the topStateMode to set
	 */
	public void setTopStateMode(TopStateMode topStateMode) {
		this.topStateMode = topStateMode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IAnalysisConfiguration#getUnknownTransitionMode()
	 */
	public UnknownTransitionMode getUnknownTransitionMode() {
		return unknownTransMode;
	}

	/**
	 * @param unknownTransMode
	 *            the unknownTransMode to set
	 */
	public void setUnknownTransitionMode(UnknownTransitionMode unknownTransMode) {
		this.unknownTransMode = unknownTransMode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IAnalysisConfiguration#getMaxDepth()
	 */
	public int getMaxDepth() {
		return maxDepth;
	}

	/**
	 * 
	 * @param maxDepth
	 *            the maxDepth to set
	 */
	public void setMaxDepth(int maxDepth) {
		this.maxDepth = maxDepth;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.unisb.prog.typestate.IAnalysisConfiguration#logUnknownSpecification()
	 */
	public boolean logUnknownSpecification() {
		return logNoSpec;
	}
	
	/**
	 * @param on true if logging of no specs should be enabled
	 */
	public void setLogUnknownSpec(boolean on) {
		logNoSpec = on;
	}

	public boolean walkerUseOnlyOneWorklist() {
		return walkerUseOnlyOneWorklist;
	}
	
	public void setWalkerUseOnlyOneWorklist(boolean val)  {
		this.walkerUseOnlyOneWorklist = val;
	}
}
