package de.unisb.prog.typestate;

import java.util.Enumeration;

import org.apache.log4j.Appender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

/**
 * Used to provide static access to the logger (avoids adding the logger to every and each method)
 * 
 * @author mikerofone
 * 
 */
public class AnalysisLogger {

	private static AnalysisLogger	instance;

	private static int				trace	= Level.TRACE.toInt(), debug = Level.DEBUG.toInt(),
			info = Level.INFO.toInt(), warn = Level.WARN.toInt(), error = Level.ERROR.toInt();

	private int						curLevel;

	public static AnalysisLogger getInstance() {
		if (instance == null) {
			instance = new AnalysisLogger();
		}
		return instance;
	}

	Logger	log;
	String	prefix, indent;
	char	tag;

	private AnalysisLogger() {
		prefix = indent = "";
		tag = '';
	}

	private void defaultLogger() {
		log = Logger.getRootLogger();
		log.setLevel(Level.ALL);
		curLevel = Level.ALL.toInt();
		// log.info("-- Initialization of default logger (none specified) --");
		setIndentation(0);
	}

	public static void setLogger(Logger log) {
		if (log == null) {
			// No logger given, so use some standard logger
			getInstance().defaultLogger();
		} else {
			getInstance().log = log;
			getInstance().curLevel = log.getLevel().toInt();
		}
	}

	public Logger getLogger() {
		return instance.log;
	}

	public boolean trace() {
		return trace >= curLevel;
	}

	public void trace(String message) {
		log.trace(prefix + message);
	}

	public void trace(String message, char tag) {
		log.trace(indent + tag + " " + message);
	}

	public boolean debug() {
		return debug >= curLevel;
	}

	public void debug(String message) {
		log.debug(prefix + message);
	}

	public void debug(String message, char tag) {
		log.debug(indent + tag + " " + message);
	}

	public boolean info() {
		return info >= curLevel;
	}

	public void info(String message) {
		log.info(prefix + message);
	}

	public void info(String message, char tag) {
		log.info(indent + tag + " " + message);
	}

	public boolean warn() {
		return warn >= curLevel;
	}

	public void warn(String message) {
		log.warn(prefix + message);
	}

	public void warn(String message, char tag) {
		log.warn(indent + tag + " " + message);
	}

	public boolean error() {
		return error >= curLevel;
	}

	public void error(String message) {
		log.error(prefix + message);
	}

	public void error(String message, char tag) {
		log.error(indent + tag + " " + message);
	}

	public Level getLevel() {
		return log.getLevel();
	}

	public void setLevel(Level lvl) {
		log.setLevel(lvl);
		curLevel = lvl.toInt();
	}

	@SuppressWarnings("unchecked")
	public void setFormat() {
		for (Enumeration<Appender> a = log.getAllAppenders(); a.hasMoreElements();) {
			a.nextElement().setLayout(new PatternLayout("%-5p %d{HH:mm:ss,SSS}:%m%n"));
		}
	}

	public void setIndentation(int level) {
		if (level < 0) {
			prefix = indent = "";
			tag = '';
		} else {
			indent = String.format("%" + (level * 3 + 2) + "s", " ");
			prefix = indent + tag + " ";
		}
	}

	public void setTag(char pref) {
		tag = pref;
		prefix = indent + tag + " ";
	}
}
