package de.unisb.prog.typestate;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Level;

import de.unisb.prog.jfirm.ir.MethodGraph;
import de.unisb.prog.jfirm.ir.Mode;
import de.unisb.prog.jfirm.ir.Node;
import de.unisb.prog.jfirm.ir.NodeVisitor;
import de.unisb.prog.jfirm.ir.Phi;
import de.unisb.prog.jfirm.ir.VCGDumper;
import de.unisb.prog.jfirm.ir.ValueNode;
import de.unisb.prog.jfirm.ir.Type.ClassType;
import de.unisb.prog.typestate.util.Types;

public class FirmConnector {

	/**
	 * Walks over a jFirm method graph and puts all found Reference-type Phis into the nodeIterator() of the
	 * corresponding Block node.
	 *
	 * @author mikerofone
	 *
	 */
	private  class PhiFinder implements NodeVisitor { // TODO use me

		public void init(int range) {}
		public void post(Node node) {}

		public void pre(Node node) {
			if (node instanceof Phi && ((ValueNode) node).getMode() == Mode.A) {
				node.getBlock().add((ValueNode)node);
			}
		}

	}


	
	private static final char			FIRMCONNECTOR_LOG_TAG	= 'F';
	private static FirmConnector		instance;
	private static AnalysisLogger		log						= AnalysisLogger.getInstance();
//	private IInputDataSet				inputSet;
	private Set<MethodGraph>			alreadyBuilt;

	private FirmConnector() {
	}

	public void init(IInputDataSet inputSet) {
//		this.inputSet = inputSet;
		alreadyBuilt = new HashSet<MethodGraph>();
		log = AnalysisLogger.getInstance();
	}

	public static FirmConnector getInstance() {
		if (instance == null) {
			instance = new FirmConnector();
		}
		return instance;
	}

	public MethodGraph getFirmGraph(ClassType.Method method) {
		String className = method.getOwner().getName();
		String signature = Types.getSignature(method);
		int delimIndex = signature.indexOf('(');
		String methodName = signature.substring(0, delimIndex);
		String params = signature.substring(delimIndex);
		if (className.startsWith("[")) {
			// Arrays do not have "code"
			log.debug("Ignoring request to load jFirm graph for " + className + "." + signature
					+ " because there is none for arrays", FIRMCONNECTOR_LOG_TAG);
			return null;
		} else {
			return getFirmGraph(className, methodName, params);
		}
	}

	public MethodGraph getFirmGraph(String className, String methodName, String params) {

		// FIXME how does jFirm get the bytecode if I don't provide it?!?
		// // Load class
		// InputStream bytecode = inputSet.getBytecodeStream(className);
		// if (bytecode == null) {
		// // class not available
		// return null;
		// }
		// cons.parseClass(className, bytecode);

		try {
			
			ClassType classType = ClassType.getByName(className);
			if (classType == null) {
				throw new ClassNotFoundException("Could not find classType for " + className);
			}
			MethodGraph methodGraph = null;
			try {
				methodGraph = classType.getMethod(methodName, params, false).getGraph();
			} catch (IllegalArgumentException e) {
				// Method not found. Try static modifier:
				methodGraph = classType.getMethod(methodName, params, true).getGraph();
			}

			
			
			if (methodGraph == null) {
				if (log.debug())
					log.debug("Method " + className + "." + methodName + " was not found.",
							FIRMCONNECTOR_LOG_TAG);
				return null;
			}
			if (!alreadyBuilt.contains(methodGraph)) {
				methodGraph.visit(new PhiFinder());
			}
			if (Level.DEBUG.isGreaterOrEqual(log.getLogger().getLevel())) {
				VCGDumper.dump(methodGraph);
			}
			return methodGraph;
		} catch (IllegalArgumentException e) {
			// MethodGraph not found
			if (log.debug())
				log
						.debug("Method " + className + "." + methodName + " was not found.",
								FIRMCONNECTOR_LOG_TAG);
		} catch (UnsupportedOperationException e) {
			// Unknown opcode
			if (log.warn())
				log.warn("jFirm could not build graph for method " + className + "." + methodName + ": "
						+ e.getMessage(), FIRMCONNECTOR_LOG_TAG);
		} catch (Throwable e) {
			// Some other error
			if (log.warn())
				log.warn("jFirm could not build graph for method " + className + "." + methodName + ": " + e,
						FIRMCONNECTOR_LOG_TAG);
		}
		// Some error occurred
		return null;

	}

}
