package de.unisb.prog.typestate;

import java.util.List;

/**
 * Running the analysis will return an instance of this class as result. The following information is returned
 * to the caller:<br>
 * <ul>
 * <li>An int determining how many violations were found.</li>
 * <li>A <code>List</code> of {@link IViolationPoint}s which each give a class name, line number and violation
 * description for every detected violation. The entries in the list are ordered by increasing line number.</li>
 * </ul>
 * 
 * @author mikerofone
 * 
 */
public interface IAnalysisResults {

	/**
	 * @return The number of violations which were found in the run.
	 */
	public int getNumberOfViolations();

	/**
	 * Returns a list containing all the {@link IViolationPoint}s found in the specified class.
	 * @param The fully qualified class name
	 * @return A list containing the violation points ordered by their appearance in the code (&rarr;
	 *         increasing line number in source code)
	 */
	public List<IViolationPoint> getViolationList(String className);
}
