package de.unisb.prog.typestate;

import java.io.Serializable;

import org.apache.log4j.Logger;

/**
 * This interface provides the necessary mechanisms for caching analysis results permanently. It just provides
 * mechanisms for writing and reading {@link Serializable} objects from and to some directory.<br>
 * The cache provider can output messages to a <code>log4j</code> logger given by a call to {@link setLogger}.
 * 
 * @author mikerofone
 * 
 */
public interface ICacheProvider {

	/**
	 * Sets a <code>log4j</code> logger where messages related to caching can be output.
	 * 
	 * @param logger
	 *            The logger to use
	 */
	public void setLogger(Logger logger);

	/**
	 * Determines if there is a cached version of specified object.
	 * 
	 * @param name
	 *            The name of the {@link Serializable} object which is requested
	 * @return True if there is a cached version available, false if not
	 */
	public boolean hasCachedEntry(String name);

	/**
	 * Returns the cached {@link Serializable} with the specified name.
	 * 
	 * @param name
	 *            The name of the object to read
	 * @return The cached object if available, or <code>null</code> if not
	 */
	public Serializable getCachedEntry(String name);

	/**
	 * Stores a {@link Serializable} object and labels it with the given name.
	 * 
	 * @param name
	 *            The name with which the object can be requested later
	 * @param object
	 *            The {@link Serializable} object to put into the cache
	 */
	public void cacheObject(String name, Serializable object);
}
