package de.unisb.prog.typestate;

import java.util.Set;

/**
 * Edges in the specification automata are marked with <code>ITransition</code>s. The following information is
 * stored:
 * <ul>
 * <li>The name of the source state</li>
 * <li>The name of the destination state</li>
 * <li>The set of {@link IMethodIdentifier}s indicating which method calls fire this transition</li>
 * </ul>
 * 
 * @author mikerofone
 * 
 */
public interface ITransition {

	/**
	 * @return the name of the source state
	 */
	public String getSourceName();

	/**
	 * @return the name of the destination state
	 */
	public String getDestinationName();

	/**
	 * Returns the marking of the transition. The marking names a set of method signatures, meaning that
	 * calling this method on the underlying object instance, it will perform a transition from the current to
	 * the destination state.
	 * 
	 * @return The set of {@link IMethodIdentifier}s naming the relevant methods for this transition
	 */
	public Set<IMethodIdentifier> getMarking();

}
