package de.unisb.prog.typestate;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A special <code>null</code> state for marking instances as possibly being null
 * 
 * @author mikerofone
 * 
 */
public class NullState implements IState {

	private static Set<IState>				nullExn;
	public static final IState nullDeref = new State("<null deref>", true, null); 
	private Specification					spec;

	public NullState() {
		this.spec = null;
	}

	public NullState(Specification spec) {
		this.spec = spec;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getName()
	 */
	public String getName() {
		return "<null>";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getTransitions()
	 */
	public Map<String, Set<IState>> getTransitions() {
		return Collections.emptyMap();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#isBad()
	 */
	public boolean isBad() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#performTransition(java.lang.String)
	 */
	public Set<IState> performTransition(String action) {
		if (nullExn == null) {
			nullExn = new HashSet<IState>(1);
			nullExn.add(nullDeref);
		}
		return nullExn;
	}

	public String toString() {
		return getName();
	}

	/**
	 * Checks if a given state stands for the instance being <code>null</code>
	 * 
	 * @param state
	 *            the state
	 * @return true if the given {@link IState} is a {@link NullState} instance
	 */
	public static boolean isNull(IState state) {
		return state instanceof NullState;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getTypeSpecification()
	 */
	public Specification getTypeSpecification() {
		return spec;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.spec == null) ? 0 : this.spec.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj instanceof NullState) {
			NullState other = (NullState) obj;
			return this.spec == null ? other.spec == null : this.spec.equals(other.spec);
		} else {
			return false;
		}

	}

}
