package de.unisb.prog.typestate;

import java.io.IOException;
import java.util.Enumeration;

import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import de.unisb.prog.typestate.violation.IViolationInfo;

public class TestMain {

	@SuppressWarnings("unchecked")
	public static void main(String[] args) {

		Logger log = Logger.getRootLogger();
		for (Enumeration<Appender> apps = log.getAllAppenders(); apps.hasMoreElements();) {
			apps.nextElement().setLayout(new PatternLayout("%-5p %d{DATE}: %m%n"));
		}
		try {
			log.addAppender(new FileAppender(new PatternLayout("%-5p %d{DATE}: %m%n"), "./log", false));
		} catch (IOException e) {
			log.debug("Cannot create logfile, only logging to console");
		}
		log.setLevel(Level.INFO);
		IAnalysis ana = new AnalysisRunner();

		TestInputDataSet inputData = TestInputDataSet.generateExampleInput();

		// **
		// RUN ANALYSIS
		// **

		IAnalysisResults results = ana.run(inputData, log);

		// **
		// OUTPUT RESULTS
		// **

		System.out.println("\n--- RESULTS: ---");
		if (results.getViolationList("de/unisb/prog/typestate/TestClass") != null) {
			for (IViolationPoint vp : results.getViolationList("de/unisb/prog/typestate/TestClass")) {
				System.out.println(String.format("[%s]%s@%d(%s):", vp.getSeverity(), vp.getClassName(), vp
						.getLineNumber(), vp.getLocation().toString()));
				for (IViolationInfo vi : vp.getInfo()) {
					System.out.println(vi.getType() + ":" + vi.text() + "\n");
				}
			}
		}
	}

	
}
