package de.unisb.prog.typestate;

/**
 * This class is intended to be a replacement for Java annotations, because they can be only applied to
 * definitions and are currently not supported by jFirm.<br>
 * <br>
 * It provides numerous methods which do nothing. The analysis will recognize calls to these methods and
 * evaluate them appropriately. See the description of the methods for details.
 * 
 * @author mikerofone
 * 
 */
public class TypestateHelper {

	/**
	 * Tells the analysis to not assume the <code>reference</code> being in state <code>stateName</code> after
	 * the line containing this call.
	 * 
	 * @param reference
	 *            the affected reference
	 * @param stateName
	 *            the name of the state to ignore
	 */
	public static void state_ignore(Object reference, String stateName) {
		// do nothing
	}

	/**
	 * Tells the analysis to assume the <code>reference</code> being in state <code>stateName</code> after the
	 * line containing this call.
	 * 
	 * @param reference
	 *            the affected reference
	 * @param stateName
	 *            the name of the state to assume
	 */
	public static void state_assume(Object reference, String stateName) {
		// do nothing
	}

	/**
	 * Tells the analysis to set the state of <code>reference</code> to be exactly <code>stateName</code>
	 * after the line containing this call.
	 * 
	 * @param reference
	 *            the affected reference
	 * @param stateName
	 *            the name of the state to set
	 */
	public static void state_set(Object reference, String stateName) {
		// do nothing
	}

	/**
	 * Generates an INFO log message with the current state of <code>reference</code> together with a custom
	 * message.
	 * 
	 * @param reference
	 *            the reference to print the state for
	 * @param message
	 *            a custom message to help recognizing this output
	 */
	public static void state_print(Object reference, String message) {
		// do nothing
	}

	/**
	 * Tells the analysis to ignore any occurring typestate violations from now on. Use in combination with
	 * {@link #ignoreErrors_end()} to disable violation reporting for example in a block.<br>
	 * <strong>Important:</strong> Always make sure that a block cannot escape the call to
	 * {@link #ignoreErrors_end()} (exceptions, <code>return</code>s, <code>break</code>s etc.). This can be
	 * best achieved by using a <code>finally</code> block:<br>
	 * <br>
	 * <code>TypestateHelper.ignoreErrors_begin();<br>
	 * try {<br>
	 * &nbsp;&nbsp;&lt;the program code for which error checking should be disabled&gt;<br>
	 * } finally {<br>
	 * &nbsp;&nbsp;TypestateHelper.ignoreErrors_end();<br>
	 * }</code>
	 */
	public static void ignoreErrors_begin() {
		// do nothing
	}

	/**
	 * Tells the analysis to stop ignoring typestate violations and return back to normal mode. Use in
	 * combination with {@link #ignoreErrors_begin()} to disable violation reporting for example in a block.
	 */
	public static void ignoreErrors_end() {
		// do nothing
	}
}
