package de.unisb.prog.typestate;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A special <code>null</code> state for marking instances as possibly being null
 * 
 * @author mikerofone
 * 
 */
public class UnknownState implements IState {

	private static UnknownState	instance;
	private static Set<IState>	sameTransition;

	private UnknownState() {
		sameTransition = new HashSet<IState>(1);
		sameTransition.add(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getName()
	 */
	public String getName() {
		return "<??>";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getTransitions()
	 */
	public Map<String, Set<IState>> getTransitions() {
		return Collections.emptyMap();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#isBad()
	 */
	public boolean isBad() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#performTransition(java.lang.String)
	 */
	public Set<IState> performTransition(String action) {
		return sameTransition;
	}

	public String toString() {
		return getName();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.IState#getTypeSpecification()
	 */
	public Specification getTypeSpecification() {
		return null;
	}
	
	public static UnknownState getInstance() {
		if (instance == null) {
			instance = new UnknownState();
		}
		return instance;
	}

}
