package de.unisb.prog.typestate;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * This state inidcates that the outcome of a transition was unspecified (e.g. the requested transition was
 * not contained in the given specification).
 * 
 * @author mikerofone
 * 
 */
public class UnspecifiedState implements IState {

	private Specification	spec;
	private HashSet<IState>	thisSet;

	public UnspecifiedState(Specification spec) {
		this.spec = spec;
	}

	public String getName() {
		return "<unspec>";// + (spec == null ? ">" : "(" + spec.getTypeName()) + ")>";
	}

	public Map<String, Set<IState>> getTransitions() {
		return Collections.emptyMap();
	}

	public Specification getTypeSpecification() {
		return spec;
	}

	public boolean isBad() {
		return true;
	}

	public Set<IState> performTransition(String action) {
		if (thisSet == null) {
			thisSet = new HashSet<IState>();
			thisSet.add(this);
		}
		return thisSet;
	}
	
	@Override
	public String toString() {
		return getName();
	}
}
