package de.unisb.prog.typestate.ir;

import java.util.HashMap;
import java.util.Iterator;

import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.Block;
import de.unisb.prog.jfirm.ir.Mode;
import de.unisb.prog.jfirm.ir.ValueNode;
import de.unisb.prog.typestate.ir.FirmWalker.Globals;
import de.unisb.prog.typestate.util.NodeHelper;

public class BlockHandler {

	Globals	g;

	public void init(Globals g) {
		this.g = g;
	}

	/**
	 * Initialize the Reference Phi Nodes in the Block to their correct state, depending on the entry path
	 * taken into the block (as this sets the currently valid input of the phis)
	 * 
	 * @param toBlock
	 * @param prevBlock
	 */
	public void handleBlock(Block toBlock, ValueNode firstBlockNode, Block prevBlock) {
		// throw new IllegalStateException("not implemented");

		// (PATHSENSITIVITY) determine the index i of the entry path

		HashMap<ValueNode, Reference> tempStates = new HashMap<ValueNode, Reference>();

		boolean firstNodeIsMemPhi = firstBlockNode.getMode() == Mode.M;

		// for every phi in block
		for (Iterator<ValueNode> it = toBlock.nodesIterator(); it.hasNext();) {
			ValueNode phi = it.next();
			Reference newPhiState = new Reference(NodeHelper.getType(phi));

			// merge state of...
			// ...(PATHSENSITIVITY) input i into current state stored for input i
			// ... all inputs into current state
			for (int i = 0, n = phi.getN(); i < n; i++) {
				ValueNode phiInput = NodeHelper.getInput(phi, i);
				ContextAndNode phiInputNode = new ContextAndNode(g.context, phiInput);
				StateTuple inState;
				// If input is from another block, use this block's out memory to get the correct state for
				// that object.
				// Use the current snapshot instead of the input memory if there is a phi loop
				if (firstNodeIsMemPhi && !phi.equals(phiInput)) {
					Block inputMemBlock = firstBlockNode.getIn(i).getBlock();
					inState = g.methodMem.readState(phiInputNode, inputMemBlock);
				} else {
					inState = g.methodMem.readState(phiInputNode);
				}

				// because there is no path distinction, some inputs might be unknown and return null
				if (inState != null) {
					newPhiState.update(inState, false);
				}
			}

			// and store at a temporary (!!) place
			tempStates.put(phi, newPhiState);
		}

		// Now apply temporary state for every phi in block
		for (Iterator<ValueNode> it = toBlock.nodesIterator(); it.hasNext();) {
			ValueNode phi = it.next();

			// apply temporary state info
			ContextAndNode phiNode = new ContextAndNode(g.context, phi);

			g.methodMem.putAsReference(phiNode, tempStates.get(phi));
		}
		g.methodMem.printDumpCurrMem();
	}
}
