package de.unisb.prog.typestate.ir;

import de.unisb.prog.jfirm.ir.Block;

public class BlockMemory {

	public static enum Mem {
		IN, OUT
	}

//	private ArrayList<HashMap<ContextAndNode, Integer>>	inPhis;
//	private ArrayList<HashMap<ContextAndNode, Integer>>	outPhis;
//
//	private HashMap<ContextAndNode, Set<Integer>>		phiInputsTaken;

	private MemorySnapshot								inSnapshot;
	private MemorySnapshot								outSnapshot;

	private Block										block;

	public BlockMemory(Block block) {
		if (block == null) {
			throw new IllegalArgumentException("BlockMemory(block==null) not allowed");
		}

		this.block = block;
//		phiInputsTaken = new HashMap<ContextAndNode, Set<Integer>>();
	}

	/** Gets the block this memory stores information for */
	public Block getBlock() {
		return block;
	}

	/** Returns whether there already is an IN/OUT memory for this block */
	public boolean hasMem(Mem which) {
		return (which == Mem.IN ? inSnapshot : outSnapshot) != null;
	}

	/** Creates an empty IN/OUT memory for this block */
	public void createMem(Mem which) {
		assert (which == Mem.IN ? inSnapshot : outSnapshot) == null : "BlockMemory for " + block
				+ " already has " + which + "-Memory: " + inSnapshot;
		switch (which) {
		case IN:
			inSnapshot = new MemorySnapshot();
//			inPhis = new ArrayList<HashMap<ContextAndNode, Integer>>();
			break;
		case OUT:
			outSnapshot = new MemorySnapshot();
//			outPhis = new ArrayList<HashMap<ContextAndNode, Integer>>();
			break;
		}
	}

	/** Returns a deep copy of the IN/OUT memory of this block */
	public MemorySnapshot getDeepMemCopy(Mem which) {
		MemorySnapshot mem = (which == Mem.IN ? inSnapshot : outSnapshot);
		assert mem != null : which + "-Memory not inited";
		return mem.deepCopy();
	}

	/**
	 * Performs an update of the IN/OUT memory of this block with the data of the IN/OUT memory of the
	 * argument BlockMemory
	 * 
	 * @param toWhich
	 *            which memory of this block is to be updated
	 * @param fromMem
	 *            from which BlockMemory will data be merged into the current BlockMemory
	 * @param fromWhich
	 *            from which memory of <code>fromMem</code> will the data be taken
	 * @return whether the <code>toWhich</code> memory of this block changed or not
	 */
	public boolean updateMemory(Mem toWhich, BlockMemory fromMem, Mem fromWhich) {
		MemorySnapshot newStates = (fromWhich == Mem.IN ? fromMem.inSnapshot : fromMem.outSnapshot);
		return updateMemory(toWhich, newStates);
	}

	/** Performs an update of the IN/OUT memory of this block with the given memory snapshot. */
	public boolean updateMemory(Mem which, MemorySnapshot fromMem) {
		if (!hasMem(which)) {
			// First definition of this memory
			createMem(which);
		}
		MemorySnapshot mem = (which == Mem.IN ? inSnapshot : outSnapshot);

		return mem.merge(fromMem);

	}

	/* -------------------------- */
	/* DIRECT IN/OUT MANIPULATION */
	/* -------------------------- */
	
	public void setMemory(Mem which, MemorySnapshot memSnapshot) {
		switch (which) {
		case IN:
			inSnapshot = memSnapshot;
			break;
		case OUT:
			outSnapshot = memSnapshot;
			break;
		}

	}
	
	// /** Returns whether the IN/OUT memory of this block contains a mapping for the given node */
	// public boolean contains(Mem which, ContextAndNode node) {
	// return (which == Mem.IN ? inSnapshot : outSnapshot).getType(node) != null;
	// }
	//
	// /** Returns one concrete entry from the specified memory */
	// public MutableStateTuple getState(Mem fromWhich, ContextAndNode forNode) {
	// MemorySnapshot mem = (fromWhich == Mem.IN ? inSnapshot : outSnapshot);
	// assert mem != null : fromWhich + "-Memory of " + block + " not inited";
	// MutableStateTuple nodeState = mem.get(forNode);
	// if (nodeState == null) {
	// throw new IllegalArgumentException(fromWhich + "-Memory of " + block + " does not contain "
	// + forNode);
	// }
	// return nodeState;
	// }
	//
	// /** Sets the state of one concrete entry in the specified memory */
	// public void putState(Mem toWhich, ContextAndNode node, MutableStateTuple state) {
	// MemorySnapshot mem = (toWhich == Mem.IN ? inSnapshot : outSnapshot);
	// assert mem != null : toWhich + "-Memory of " + block + " not inited";
	// mem.put(node, state);
	// }

	public MemorySnapshot getView(Mem which) {
		return which == Mem.IN ? inSnapshot : outSnapshot;
	}

	public String toString() {
		return "-BlockMemory " + block + "-";
	}

	public String toDetailedString() {
		StringBuilder sb = new StringBuilder("-BlockMemory " + block + "-\n");
		if (inSnapshot != null) {
			sb.append(" IN :\n");
			sb.append(inSnapshot.toString());
		}
		if (outSnapshot != null) {
			sb.append(" OUT:\n");
			sb.append(outSnapshot.toString());
		}
		return sb.toString();
	}

}
