/**
 * 
 */
package de.unisb.prog.typestate.ir;

import java.util.HashSet;
import java.util.Set;

import de.unisb.prog.jfirm.ir.Type;

/**
 * @author mikerofone
 * 
 */
public class FieldReference extends Reference {

	private Set<FieldID>	fields;

	public FieldReference(Type type) {
		super(type);
		this.fields = new HashSet<FieldID>();
	}

	public FieldReference(Reference copyFrom) {
		super(copyFrom);
		this.fields = new HashSet<FieldID>();
	}

	public FieldReference(FieldReference copyFrom) {
		super(copyFrom);
		this.fields = new HashSet<FieldID>(copyFrom.fields);
	}

	public void addField(FieldID field) {
		this.fields.add(field);
	}

	public Set<FieldID> getFields() {
		return fields;
	}

	@Override
	public void unregistered(AllocSite site) {
		// if registered is down to 0, remove all references to the fields to help garbage collection
		// (perhaps)
		super.unregistered(site);
		if (this.ptsTargets.size() == 0)
			fields.clear();
	}

	@Override
	public String toString() {
		return super.toString() + " - " + fields.size() + " fields: " + fields;
	}

	@Override
	protected Object clone() {
		Object clone = super.clone();
		((FieldReference) clone).fields = new HashSet<FieldID>(this.fields);
		return clone;
	}
}
