package de.unisb.prog.typestate.ir;

import java.util.HashSet;
import java.util.Set;

import de.unisb.prog.jfirm.ir.Type;
import de.unisb.prog.typestate.IState;

/**
 * A mutable version of {@link StateTuple}.
 * 
 * @see StateTuple
 * @author mikerofone
 * 
 */
public class MutableStateTuple extends StateTuple {

	// Fields from StateTuple
	// protected Set<IState> typeState;
	// protected boolean containsTop;

	/** Creates a new empty {@link MutableStateTuple}. */
	public MutableStateTuple(Type type) {
		super(type);
	}

	/** Creates a mutable clone of the argument {@link StateTuple} */
	protected MutableStateTuple(StateTuple copyFrom) {
		super(copyFrom);
	}

	/** Perform weak or strong updates and return whether this changed the state */
	public boolean update(StateTuple fromTuple, boolean strongUpdate) {
		// merge the TOP-information from fromTuple
		boolean changed = false;
		if (strongUpdate || fromTuple.containsTop) {
			// overwrite local TOP state
			changed = this.containsTop != fromTuple.containsTop;
			this.containsTop = fromTuple.containsTop;
		}
		return update(fromTuple.getStateSet(), strongUpdate) || changed;
	}

	/** Perform weak or strong updates and return whether this changed the state */
	public boolean update(Set<IState> newState, boolean strongUpdate) {
		boolean updated = false;

		if (strongUpdate) {
			updated = !typeState.equals(newState);
			HashSet<IState> newTypeState = new HashSet<IState>(newState);
			typeState.clear();
			typeState = newTypeState;
		} else {
			updated = typeState.addAll(newState);
		}

		return updated;
	}

	/** Removes all states */
	public void clearTypestate() {
		typeState.clear();
		this.containsTop = false;
	}

	/** Adds a single state to the tuple (weak update) */
	public boolean addState(IState state) {
		return typeState.add(state);
	}

	/** Strong update from another StateTuple */
	public void overwrite(StateTuple inState) {
		overwrite(inState.getStateSet());
	}

	/** Strong update with state set */
	public void overwrite(Set<IState> newState) {
		typeState.clear();
		typeState.addAll(newState);
	}

	/** Removes the given state */
	public boolean removeState(IState state) {
		return typeState.remove(state);
	}
}
