package de.unisb.prog.typestate.ir;

import java.util.HashSet;
import java.util.Set;

import de.unisb.prog.analysis.Context;
import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.DataMemExec;
import de.unisb.prog.jfirm.ir.NewObject;
import de.unisb.prog.jfirm.ir.Type;
import de.unisb.prog.jfirm.ir.ValueNode;
import de.unisb.prog.typestate.IState;
import de.unisb.prog.typestate.Specification;
import de.unisb.prog.typestate.UnknownState;
import de.unisb.prog.typestate.ir.FirmWalker.Globals;
import de.unisb.prog.typestate.util.NodeHelper;

public class NewHandler {

	/* ---------------------------- */
	/* NEW HANDLER */
	/* ---------------------------- */

	private Globals	g;

	public void init(Globals g) {
		this.g = g;
	}

	/**
	 * Updates the target memory of a NewObject Node with the initial state given by the specification<br>
	 * TODO: Must this create a new separate instance if run more than once?
	 */
	public ValueNode evaluateNew(NewObject newNode) {

		// Find specification for the type
		Type type = NodeHelper.getType(newNode);
		Specification spec = g.specRep.getSpecification(type);
		Set<IState> initState = new HashSet<IState>();

		Context currContext = g.context;
		ContextAndNode newCNode = new ContextAndNode(currContext, newNode);
		if (spec == null) {
			// Specification unknown, so put unknown state
			g.vioStore.addMissingSpec(type.toString(), newCNode, newCNode);
			initState.add(UnknownState.getInstance());
		} else {
			initState.add(spec.getStartingState());
		}

		MutableStateTuple newTuple = new AllocSite(newCNode, type);
		newTuple.overwrite(initState);
		// FIXME must create a new allocation site which is distinct from everything.... or not?
		// Kind of, if a) path sensitive and b) aliasing information can be handled without the need of the
		// refs being connected to their source alloc site
		
		// create new allocsite
		g.methodMem.createAllocSite(newCNode, newTuple);

		return newNode.getValue(DataMemExec.MEM);
	}
}
