package de.unisb.prog.typestate.ir;

import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.Param;
import de.unisb.prog.jfirm.ir.Type;
import de.unisb.prog.typestate.Specification;
import de.unisb.prog.typestate.TopState;
import de.unisb.prog.typestate.UnknownState;
import de.unisb.prog.typestate.ir.FirmWalker.Globals;
import de.unisb.prog.typestate.util.NodeHelper;

public class ParamHandler {
	/* ---------------------------- */
	/* PARAM HANDLER */
	/* ---------------------------- */

	private Globals	g;

	public void init(Globals g) {
		this.g = g;
	}

	/**
	 * Gets the starting state of the type from the specRepository, calls all available constructors and
	 * stores it in the first memory
	 * 
	 * @param param
	 *            the Param Node which is to be inited
	 */
	public void initParam(Param param) {
		Type paramType = NodeHelper.getType(param);
		Specification spec = g.specRep.getSpecification(paramType);
		MutableStateTuple paramInitedState = new MutableStateTuple(paramType);
		ContextAndNode paramNode = new ContextAndNode(g.context, param);
		if (spec == null) {
			// Specification unknown, so put unknown state
			g.vioStore.addMissingSpec(paramType.toString(), paramNode, paramNode);
			paramInitedState.addState(UnknownState.getInstance());
		} else {
			// spec available

			// initialize with TOP
			paramInitedState.addState(TopState.getTop(spec, g.analysisConfig.getTopStateMode()));

		}
		// extend the first memory
		
		// put the allocation site for the param into the memory
		g.methodMem.createAllocSite(paramNode, paramInitedState);

	}
}
