package de.unisb.prog.typestate.ir;

import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.Mode;
import de.unisb.prog.jfirm.ir.ValueNode;
import de.unisb.prog.typestate.ir.FirmWalker.Globals;
import de.unisb.prog.typestate.util.NodeHelper;

public class ReturnHandler {

	private static final char	RETURNHANDLER_LOG_TAG	= 'R';
	private Globals				g;

	public void init(Globals g) {
		this.g = g;
	}

	public ValueNode evaluateReturn(ValueNode returnNode) {
		// only do something if...

		// a) this return node actually has an input (input0 is memory)
		if (returnNode.getN() > 1) {

			ContextAndNode input = new ContextAndNode(g.context, NodeHelper.getInput(returnNode,1));

			// b) the input is a reference
			if (NodeHelper.getDataNode(input.getNode()).getMode() == Mode.A) {

				// c) the current memory can store the return value (e.g. is not the top-level call)
				ContextAndNode returnTarget = g.methodMem.getReturnNode();
				if (returnTarget != null) {

					// Get the state of the return input
					StateTuple retState = g.methodMem.readState(input);
					if (retState == null) {
						g.log.error("State of input " + input + " of " + returnNode + " used for "
								+ returnTarget + " is unknown. Propagating BOTTOM state information",
								RETURNHANDLER_LOG_TAG);
//						// create an empty state tuple
//						retState = new StateTuple();
						return null;
					}

					// Update return state information in the memory
					g.methodMem.updateReturnState(retState);

				}
			}
		}
		// return the given Return node because it is the next "memory" node itself
		return returnNode;
		
//		//return null because the trace ends here
//		return null;
	}
}
