package de.unisb.prog.typestate.ir;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.Type;
import de.unisb.prog.typestate.IState;

/**
 * A tuple containing state information about a {@link ContextAndNode}. {@link StateTuple}s are immutable, but
 * a modifiable <strong>copy</strong> can be created with {@link StateTuple#mutableClone()}.
 * 
 * @author mikerofone
 * 
 */
public class StateTuple implements Iterable<IState> {

	Set<IState>		typeState;
//	protected ContextAndNode	source;
	boolean			containsTop;
	Type				type;

	public StateTuple(Type type) {
		typeState = new HashSet<IState>();
		this.type = type;
		this.containsTop = false;
	}
	
//	public StateTuple(ContextAndNode source) {
//		this.source = source;
//		typeState = new HashSet<IState>();
//		this.isTop = false;
//	}

	/** Cloning constructor */
	public StateTuple(StateTuple srcTuple) {
		this.typeState = new HashSet<IState>(srcTuple.typeState);
		this.containsTop = srcTuple.containsTop;
		this.type = srcTuple.type;
	}

	/** Returns an unmodifiable view of the current state */
	public Set<IState> getStateSet() {
		return Collections.unmodifiableSet(typeState);
	}
	
	public Type getType() {
		return type;
	}
	
	public void setType(Type type) {
		this.type = type;
	}

	/** Sets whether this StateTuple contains a TOP */
	public void setContainsTop(boolean top) {
		this.containsTop = top;
	}

	/** Gets whether this StateTuple contains TOP */
	public boolean containsTop() {
		return containsTop;
	}

	/** Returns a mutable copy of this StateTuple */
	public MutableStateTuple mutableClone() {
		MutableStateTuple ret = new MutableStateTuple(this);
		return ret;
	}

	/** Iterates over all states in the StateTuple */
	public Iterator<IState> iterator() {
		return typeState.iterator();
	}

	/** Returns whether the contained state set is empty or not */
	public boolean isBottom() {
		return typeState.isEmpty();
	}

	/** Returns the amount of states in this tuple */
	public int stateCount() {
		return typeState.size();
	}

	@Override
	public String toString() {
		return "<"+ (containsTop? "T" : "") + typeState + ">";
	}

	/** Returns whether all of the argument's states are contained in this one */
	public boolean contains(StateTuple other) {
		return this.typeState.containsAll(other.typeState);
	}
}
