package de.unisb.prog.typestate.ir.oldstuff;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.unisb.prog.jfirm.ir.Node;

/**
 * A class representing a trace in a method. An instance is the first action in the trace,
 * and it has a set of successor traces.
 * @author mikerofone
 *
 */
public class Trace implements Cloneable {
	private Set<Trace> next_;
	private Node firstAction_;

	public Trace(Node action) {
		firstAction_ = action;
		next_ = new HashSet<Trace>();
	}
	
	/**
	 * @return The name of the first action in this trace.
	 */
	public String getName() {
		return firstAction_.toString();
	}
	
	/**
	 * @return The first action (node visited) in the trace.
	 */
	public Node getNode() {
		return firstAction_;
	}
	
	/**
	 * Indicates if this is the last element in the trace.
	 * @return True if last, else false
	 */
	public boolean isLast() {
		return next_.isEmpty();
	}
	
	/**
	 * Adds another successor to this trace.
	 * @param newSucc The new successor trace
	 * @return True, if this add was successful, false if newSucc == null or already a successor.
	 */
	public boolean addSuccessor(Trace newSucc) {
		return newSucc == null ? false : next_.add(newSucc);
	}
	
	/**
	 * Iterate through the successor traces.
	 * @return An iterator over the successors whose iteration order is undefined.
	 */
	public Iterator<Trace> iterator() {
		return next_.iterator();
	}
	
	/**
	 * Clones a trace and all of its successors.
	 * @return A deep copy of the trace and its successors.
	 */
	public Object clone() {
		Trace temp = new Trace(firstAction_);
		temp.next_ = new HashSet<Trace>();
		for (Trace tr : next_) {
			temp.next_.add((Trace) tr.clone());
		}
		return temp;
	}

	public String toString() {
		if (!next_.isEmpty())	return firstAction_.toString() + " -> \n" + next_.iterator().next().toString();
		else return firstAction_.toString();
	}
}
