package de.unisb.prog.typestate.violation;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import de.unisb.prog.analysis.ContextAndNode;
import de.unisb.prog.jfirm.ir.DebugInfo;
import de.unisb.prog.typestate.IState;
import de.unisb.prog.typestate.ir.StateTuple;

public class CallViolationInfo implements IViolationInfo {

	private ContextAndNode		affectedAlloc;
	private Set<IState>			preState, postState;
	private String				methodName;
	private List<ISuggestion>	suggestions;

	public CallViolationInfo(ContextAndNode affectedAlloc, StateTuple preState, StateTuple postState,
			String methodName, ISuggestion sugg) {
		ArrayList<ISuggestion> suggestions = new ArrayList<ISuggestion>(1);
		suggestions.add(sugg);
		init(affectedAlloc, preState, postState, methodName, suggestions);
	}

	public CallViolationInfo(ContextAndNode affectedAlloc, StateTuple preState, StateTuple postState,
			String methodName, List<ISuggestion> suggestions) {
		init(affectedAlloc, preState, postState, methodName, suggestions);
	}

	private void init(ContextAndNode affectedAlloc, StateTuple preState, StateTuple postState,
			String methodName, List<ISuggestion> suggestions) {
		this.affectedAlloc = affectedAlloc;
		this.preState = preState.getStateSet();
		this.postState = postState.getStateSet();
		this.methodName = methodName;
		this.suggestions = suggestions;

	}

	public InfoType getType() {
		return InfoType.CALL_VIOLATION;
	}

	public Set<IState> getPreState() {
		return preState;
	}

	public Set<IState> getPostState() {
		return postState;
	}

	public String text() {
		StringBuilder sb = new StringBuilder();
		// sb.append(String.format("Instance %s performs illegal transition: %s --%s-> %s", affectedAlloc,
		// preState, methodName, postState));
		DebugInfo dbgi = affectedAlloc.getNode().getDebugInfo();
		if (dbgi == null) {
			dbgi = affectedAlloc.getNode().getProducer().getDebugInfo();
		}

		if (dbgi != null) {
			// Line information available
			sb.append(String.format("Error transition on variable defined in line %d: %s --%s-> %s", dbgi
					.getLine(), preState, methodName, postState));
		} else {
			// No debug/line info available
			sb.append(String.format("Error transition on %s: %s --%s-> %s", affectedAlloc.getNode().toString(),
					preState, methodName, postState));
		}

		if (suggestions != null && !suggestions.isEmpty()) {
			sb.append("\nFix suggestions:");
		}
		for (ISuggestion sugg : suggestions) {
			String msg = sugg.text();
			if (!"".equals(msg)) {
				sb.append("\n");
				sb.append(msg);
			}
		}
		return sb.toString();
	}
}
