package de.unisb.prog.typestate.violation;

import de.unisb.prog.typestate.IViolationPoint;

/**
 * Defines additional information for {@link IViolationPoint}s passed back to the initiator of the analysis.
 * Apart from {@link #text()}, different implementations might provide completely different methods for
 * accessing their data.
 * 
 * @author mikerofone
 * 
 */
public interface IViolationInfo {

	public enum InfoType {
		GENERIC_STRING, CALL_VIOLATION, MISSING_SPEC
	}

	/**
	 * Returns the type of the <code>IViolationInfo</code> as values of {@link IViolationInfo.InfoType}
	 * 
	 * @return the corresponding {@link IViolationInfo.InfoType}
	 */
	public InfoType getType();

	/**
	 * Every {@link IViolationInfo} implementation must implement a human-readable textual representation,
	 * returned by this method.
	 * 
	 * @return a <code>String</code> representation of the {@link IViolationInfo}
	 */
	public String text();
	
}
