package de.unisb.prog.typestate.violation;

import de.unisb.prog.jfirm.ir.ValueNode;

/**
 * Provides information about missing specification.<br>
 * <br>
 * The name of the type for which specification is missing is returned by {@link #getMissingTypeName()}.
 * 
 * @author mikerofone
 * 
 */
public class MissingSpecInfo implements IViolationInfo {

	/** Where the spec was missing */
	private ValueNode	where;

	/** Type for which no spec is available */
	private String		typeName;

	public MissingSpecInfo(ValueNode where, String typeName) {
		this.where = where;
		this.typeName = typeName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.violation.IViolationInfo#getType()
	 */
	public InfoType getType() {
		return InfoType.MISSING_SPEC;
	}

	/**
	 * @return the name of the type which is missing information
	 */
	public String getMissingTypeName() {
		return typeName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.unisb.prog.typestate.violation.IViolationInfo#text()
	 */
	public String text() {
		return (where == null ? "" : where.toString() + ": ") + "Cannot verify objects of type " + typeName + " because no specification available.";
	}

}
