package de.unisb.prog.typestate.violation;

import java.util.Set;

import de.unisb.prog.typestate.IState;

public class StateExclusionSuggestion implements ISuggestion {

	private Set<IState>	ignoreSources;
	private boolean			ignoreErrorTrans;

	/**
	 * Creates a new violation information proposing to ignore some states in the following execution
	 * 
	 * @param ignoreSources
	 *            source states which must be ignored to avoid a violation if no other method is chosen
	 * @param ignoreErrorTransitions
	 *            if <code>true</code>, transitions to error states are just thrown away (for transitions
	 *            going to error non-deterministically)
	 */
	public StateExclusionSuggestion(Set<IState> ignoreSourceStates, boolean ignoreErrorTransitions) {
		this.ignoreSources = ignoreSourceStates;
		this.ignoreErrorTrans = ignoreErrorTransitions;
	}

	public SuggestionType getType() {
		return SuggestionType.STATE_EXCLUSION;
	}

	public String text() {
		StringBuilder sb = new StringBuilder();
		if (!ignoreSources.isEmpty()) {
			sb.append("Use TypestateHelper.state_ignore(..) to ignore the following states: ");
			sb.append(ignoreSources.toString());
		}
		if (ignoreErrorTrans) {
			if (sb.length() > 0 && sb.charAt(sb.length()-1)!='\n')
				sb.append('\n');
			sb.append("Use TypestateHelper.ignoreErrors_begin/end to ignore (non-deterministic) transitions to error state.");
		}
		String msg = sb.toString();
		return "".equals(msg) ? "No analysis instruction hints available." : msg;

	}

	public Object getSuggestions() {
		// TODO Auto-generated method stub
		return null;
	}

}
